/**
 * Property Mapping:
 *     Layout.Text0 Server URL
 *     Layout.Text1 Creation Type (aka Shim) (DEPRECATED)
 *     Layout.Text2 View
 *     Layout.Text3 Login Token
 *     Layout.Text4 Hidden Url
 *     Layout.Text5 Library Read Tab (DEPRECATED)
 *     Layout.Text6 Read Tab
 *     Layout.Text7 Filter Tab (DEPRECATED)
 *     Layout.Text8 Sources Tab
 *     Layout.Text9 Settings Tab
 *     Layout.Text10 Essentials Tab (DEPRECATED)
 *     Layout.Text11 Alerts Tab (DEPRECATED)
 *     Layout.Text12 Date Format
 *     Layout.Text13 topic_name (DEPRECATED)
 *     Layout.Text14 created_before
 *     Layout.Text15 created_after
 *     Layout.Text16 folder_name (DEPRECATED)
 *     Layout.Text17 query
 *     Layout.Text18 filter_query (DEPRECATED)
 *     Layout.Text19 location
 *     Layout.Text20 project_id
 *     Layout.Text21 fingerprint
 *     Layout.Text22 smartfilterTag
 *     Layout.Text23 noise_level
 *     Layout.Text24 dateFormat
 *     Layout.Text25 topic (creation) (DEPRECATED)
 *     Layout.Text26 folder (creation) (DEPRECATED)
 *     Layout.Text27 company (signal) (DEPRECATED)
 *     Layout.Text28 country (signal) (DEPRECATED)
 *     Layout.Text29 city (signal) (DEPRECATED)
 *     Layout.Text30 email (signal) (DEPRECATED)
 *     Layout.Text31 url (signal) (DEPRECATED)
 *     Layout.Text32 stocksymbol (signal)
 *     Layout.Text33 object_id (DEPRECATED)
 *     Layout.Text34 object_type (DEPRECATED)
 **/

/**
  * QlikView uses some very old Calendar widget that does some changes to the
  * Date prototype. We monkeypatch that on top of it to call the original function
  * if more than one parameter was given.
  **/
if (_.isFunction(Date.prototype.__msh_oldSetFullYear)) {
    Date.prototype.__patched_setFullYear = Date.prototype.setFullYear;
    Date.prototype.setFullYear = function () {
        if (arguments.length > 1) {
            return Date.prototype.__msh_oldSetFullYear.apply(this, arguments);
        } else {
            return Date.prototype.__patched_setFullYear.apply(this, arguments);
        }
    };
}

function Squirro_Widget(qv) {
    var that = this;
    this.qv = qv;
    this.$qv = $(qv.Element);

    this.DEBUG = false;
    Qv.GetDocument("").GetAllVariables(function(variables) {
        _.each(variables, function (variable) {
            if (variable.name === 'SquirroDebug' && variable.value === 'true') {
                SQUIRRO = that; // Expose global to allow digging around
                that.el = null; // In case it has been rendered before
                that.DEBUG = true;
            }
        });
    });

    this.render(qv);
}

Squirro_Widget.prototype._timeFormat = 'YYYY-MM-DDTHH:mm:ss';
Squirro_Widget.prototype._iframeParams = [
    {
        name: 'created_before',
        field: 14,
        parser: _parseTime,
        error: _couldNotParseTime
    },
    {
        name: 'created_after',
        field: 15,
        parser: _parseTime,
        error: _couldNotParseTime
    },
    {
        name: 'location',
        field: 19
    },
    {
        name: 'smartfilterTag',
        field: 22,
        parser: _splitParser
    },
    {
        name: 'view',
        field: 2
    },
    {
        name: 'token',
        field: 3
    },
    {
        name: 'dateFormat',
        field: 24
    },
    {
        name: 'project_id',
        field: 20
    }
];
Squirro_Widget.prototype._queryParams = [
    {
        name: 'smartfilter',
        field: 21
    },
    {
        name: 'noise_level',
        field: 23
    },
    {
        name: 'query',
        field: 17
    }
];


Squirro_Widget.prototype._getParams = function () {
    var params = this._parseFields(this._iframeParams);

    // Tabs - Order Matters!
    var tabs = [];
    if (this.qv.Layout.Text6.text === 1) {
        tabs.push('read');
    }

    if (this.qv.Layout.Text10.text === 1) {
        tabs.push('info');
    }

    if (this.qv.Layout.Text11.text === 1) {
        tabs.push('triggers');
    }

    if (this.qv.Layout.Text8.text === 1) {
        tabs.push('sources');
    }

    if (this.qv.Layout.Text9.text === 1) {
        tabs.push('settings');
    }

    if (_.isEmpty(tabs)) {
        tabs = 'none';
    } else {
        tabs = tabs.join(',');
    }

    params.tabs = tabs;

    var query = this._getQuery();
    if (query) {
        params.query = query;
    }

    return params;
};

Squirro_Widget.prototype._parseFields = function (fields) {
    var that = this,
        ret = {};
    _.each(fields, function (field, idx) {
        var qvProp = that.qv.Layout['Text' + field.field];
        if (_.isUndefined(qvProp) || qvProp.text === '' || qvProp.text === '-') {
            return;
        }

        var value = $.trim(qvProp.text);
        if (_.isFunction(field.parser)) {
            value = field.parser.call(that, value);
        }

        if (value === false) {
            field.error.call(that, qvProp.text, that.$el);
        } else {
            ret[field.name] = value;
        }
    });
    return ret;
};

Squirro_Widget.prototype._getQuery = function () {
    var query = [],
        params = this._parseFields(this._queryParams);

    if (params.query) {
        query.push('(' + params.query + ')');
        delete params.query;
    }

    if (params.smartfilter && params.noise_level) {
        params.smartfilter += ':' + params.noise_level;
        delete params.noise_level;
    }

    _.each(params, function (param, name) {
        query.push(name + ':' + param);
    });

    query = query.join(' AND ');
    return query;
};

Squirro_Widget.prototype.render = function (qv) {
    if (qv && this.qv !== qv) {
        // Re-render if it's a new QV context…
        this.el = null;
        this.qv = qv;
        this.$qv = $(qv.Element);
    }

    if (! this.el) {
        this.el = $('<div class="squirro"><div class="errors"></div><div class="message loading"><h2>Loading Squirro&hellip;</h2><p>Please be patient for a moment.</p></div></div>');
        this.$qv.html(this.el);
        this.$el = $(this.el);
        this.$el.delegate('.close', 'click', function (e) {
            e.preventDefault();
            $(e.currentTarget).parent().remove();
        });
    }

    if (!this.url) {
        this.url = this.qv.Layout.Text4.text;
    }

    if (this.url !== this.qv.Layout.Text0.text) {
        // Server URL has changed
        if (this.$container) {
            this.$container.remove();
            this.$container = null;
        }
        this.el = null;
        this._validateUrl(this.qv.Layout.Text0.text);
        return;
    }

    this.$el.find('.errors').html('');

    var params = this._getParams();
    this._showIframe(params);
};


Squirro_Widget.prototype._validateUrl = function (url) {
    var that = this;

    $.ajax({
        url: url + '/app/bulk/iframe_isalive',
        type: 'GET',
        dataType: 'jsonp',
        jsonp: false, // Do not add ?callback to the URL
        jsonpCallback: 'jsonp',
        timeout: 5000
    }).done(function () {
        // Server is alive - set URL and render again
        that.url = url;
        that.qv.Layout.Text4.text = url;
        that.qv.Layout.SetProperty('Chart.Text.4.Content', url);
        that.render();
    }).fail(function () {
        var error = '<p>The Squirro server at <a target="_blank" href="' + encodeURI(url) + '">';
        error += _.escape(url) + '</a> was not reachable. Please check your Server URL settings in the extension properties.</p>';

        that.$el.html('<div class="message error"><h2>Squirro not reachable</h2>' + error + '</div>');
    });
};

Squirro_Widget.prototype._showIframe = function (params) {
    var link,
        url = this.url + '/app/bulk/iframe?' + $.param(params, true);

    if (this.DEBUG) {
        if (console) {
            console.log(url);
            _.each(params, function (value, key) {
                console.log(key + ' => ' + value);
            });
        }
        link = '<a target="_blank" href="' + url + '">' + url + '</a>';
    }

    if (!this.$container || this.$container.closest('html').length === 0) { // Has to be in the DOM
        if (this.DEBUG) {
            this.$container = $('<div class="iframe">' + link + '</div>');
        } else {
            this.$container = $('<iframe width="100%" height="200" frameborder="0" src="' + url + '"></iframe>');
        }
        this.$el.append(this.$container);
    } else {
        if (this.DEBUG) {
            this.$container.html(link);
        } else {
            if (url !== this.$container.attr('src')) {
                this.$container.attr('src', url);
            }
        }
    }
    this.$el.find('.loading').remove();
};

Squirro_Widget.prototype._hasText = function (prop) {
    if (prop && prop.text !== '' && prop.text !== '-') {
        return true;
    }
    return false;
};

function _splitParser(value) {
    return value.split('|');
}

function _parseTime(timeStr) {
    var format, time;
    if (this.qv.Layout.Text12) {
        format = this.qv.Layout.Text12.text;
    }
    time = moment(timeStr, format);
    if (! time.isValid()) {
        return false;
    }
    return time.format(this._timeFormat);
}

function _couldNotParseTime(value, el) {
    if (el) {
        var error = '"' + value + '" ';
        if (_.isEmpty(this.qv.Layout.Text12.text) || this.qv.Layout.Text12.text == '-') {
            error += 'could not be parsed automatically. Please specify a ';
            error += '<a href="http://go.squirro.com/QlikViewDateFormat" target="_blank">Date Format</a>.';
        } else {
            error += 'is not a date that we understand for the format ';
            error += '"' + this.qv.Layout.Text12.text + '".';
        }
        _squirroError(error);
    }
}

function _squirroError(msg) {
    // Somehow $.append doesn't work here so we emulate it.
    var $errors = $('.squirro .errors'),
        markup = $errors.html();
    markup += '<p>' + msg;
    markup += '<a href="#"" class="close" title="close">X<a>';
    markup += '</p>';
    $errors.html(markup);
}
